package nl.piraya.blog;

import com.lowagie.text.*;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
import com.lowagie.text.pdf.TextField;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Servlet that uses iText to generate a PDF. The PDF that is created will hold a couple of text fields and a submit
 * button that submits the form data to the FDFServlet.
 *
 * @author Onno Scheffers
 */
public class PDFServlet extends javax.servlet.http.HttpServlet {
   /**
    * Handles GET requests to this servlet.
    *
    * @param request An {@link HttpServletRequest} object that contains the request the client has made of the servlet.
    * @param response An {@link HttpServletResponse} object that contains the response the servlet sends to the client.
    * @exception IOException If an input or output error is detected when the servlet handles the GET request.
    * @exception ServletException If the request for the GET could not be handled.
    */
   protected void doGet(
         final HttpServletRequest request,
         final HttpServletResponse response
   ) throws ServletException, IOException {

      // Setup a buffer for the PDF data
      ByteArrayOutputStream buffer = new ByteArrayOutputStream();
      try {
         // Create the new PDF document and add some content
         Document document = new Document(PageSize.A4);
         PdfWriter writer = PdfWriter.getInstance(document, buffer);
         document.open();
         addParagraph(document);
         addFields(writer);
         addSubmitButton(writer);
         document.close();

         // Handle the response
         response.setHeader("Expires", "0");
         response.setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0");
         response.setHeader("Pragma", "public");
         response.setContentType("application/pdf");
         response.setContentLength(buffer.size());
         OutputStream os = response.getOutputStream();
         buffer.writeTo(os);
         os.flush();
         os.close();
      } catch (Exception e) {
         throw new IOException("Problem during PDF creation", e);
      }
   }

   /**
    * Adds a paragraph of text to the document.
    *
    * @param document The PDF document.
    * @throws DocumentException If something goes wrong while adding the paragraph to the PDF document.
    */
   private void addParagraph(final Document document) throws DocumentException {
      Paragraph p = new Paragraph("Please fill out the fields below and click the submit button.");
      p.setAlignment(Element.ALIGN_CENTER);
      document.add(p);
   }

   /**
    * Adds a couple of text fields to the PDF document.
    *
    * @param writer The PdfWriter to use for adding text fields to the PDF document.
    * @throws IOException If something goes wrong while adding the text fields to the document.
    * @throws DocumentException If something goes wrong while creating the required PDF elements.
    */
   private void addFields(final PdfWriter writer) throws IOException, DocumentException {
      // Add some labels
      PdfContentByte cb = writer.getDirectContent();
      cb.beginText();
      cb.setFontAndSize(BaseFont.createFont(), 12);
      cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "First Name:", 164, 687, 0);
      cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, "Last Name:", 164, 630, 0);
      cb.endText();

      // Add the text fields
      addTextField(writer, "firstName", 679);
      addTextField(writer, "lastName", 622);
   }

   /**
    * Adds a single text field to the PDF document.
    *
    * @param writer The PdfWriter to use for adding text fields to the PDF document.
    * @param fieldName The name of the field, which will be name to use for submitting the data in this text field.
    * @param y The y-position of the bottom of the text field in points, relative to the left bottom border of the page.
    * @throws IOException If something goes wrong while adding the text fields to the document.
    * @throws DocumentException If something goes wrong while creating the required PDF elements.
    */
   private void addTextField(
         final PdfWriter writer,
         final String fieldName,
         final float y) throws IOException, DocumentException {
      TextField textField = new TextField(writer, new Rectangle(170, y, 425, y + 21), fieldName);
      textField.setBorderColor(Color.BLACK);
      textField.setBackgroundColor(new GrayColor(0.9f));
      textField.setBorderWidth(1);
      textField.setBorderStyle(PdfBorderDictionary.STYLE_BEVELED);
      textField.setAlignment(Element.ALIGN_LEFT);
      textField.setOptions(TextField.REQUIRED);
      writer.addAnnotation(textField.getTextField());
   }

   /**
    * Adds a single submit button to the PDF document.
    *
    * @param writer The PdfWriter to use for adding text fields to the PDF document.
    * @throws IOException If something goes wrong while adding the text fields to the document.
    * @throws DocumentException If something goes wrong while creating the required PDF elements.
    */
   private void addSubmitButton(final PdfWriter writer) throws IOException, DocumentException {
      PushbuttonField button = new PushbuttonField(writer, new Rectangle(170, 558, 255, 586), "submit");
      button.setText("Submit");
      button.setBackgroundColor(new GrayColor(0.7f));
      button.setVisibility(PushbuttonField.VISIBLE_BUT_DOES_NOT_PRINT);
      PdfFormField submit = button.getField();
      submit.setAction(PdfAction.createSubmitForm("fdf", null, 0));
      writer.addAnnotation(submit);
   }
}
